# -*- coding: latin1 -*-
#
# Copyright 2007, 2008 Intevation GmbH, Germany, <info@intevation.de>
# 
# This file is part of mpuls WASKA (CoMPUter-based case fiLeS - 
# Web-Anwendungs-Server fuer Kompetenzagenturen).
# 
# mpuls WASKA is free software: you can redistribute it and/or modify it under
# the terms of the GNU Affero General Public License as published by the
# Free Software Foundation, either version 3 of the License, or (at your
# option) any later version.
# 
# mpuls WASKA is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Affero General Public
# License for more details.
# 
# You should have received a copy of the GNU Affero General Public
# License along with mpuls WASKA. If not, see <http://www.gnu.org/licenses/>.
# 
# mpuls WASKA has been developed on behalf of the 
# Projekttraeger im Deutschen Zentrum fuer Luft- und Raumfahrt e.V. (PT-DLR)
# within the programme Kompetenzagenturen (Durchfuehrungsphase) funded by
# the Bundesministerium fuer Familie, Senioren, Frauen und Jugend and 
# European Social Fund resources.
#
# Authors:
# Sascha L. Teichmann <teichmann@intevation.de>
#

VISIT_IGNORE_CHILDREN = 2
VISIT_CONTINUE        = None

class NodeComponent:
    
    def __init__(self):
        self.parent     = None
        self.attributes = {}
        self.attributes["name"]         = ""
        self.attributes["description"]  = ""
        self.attributes["formularname"] = ""
        self.attributes["modes"]        = ""
        self.attributes["annotation"]   = ""
        self.attributes["target"]       = ""
        self.attributes["alternative"]  = ""
        self.attributes["flags"]        = ""

    def setAttribute(self, key, value, broadcast=True):
        self.attributes[key] = value
                
    def getAttribute(self, key):
        return self.attributes.get(key, None)

    def getAnnotation(self):
        return self.getAttribute("annotation")

    def setAnnotation(self, annotation):
        return self.setAttribute("annotation", annotation)

    def getAlternative(self):
        return self.getAttribute("alternative")

    def setAlternative(self, alternative):
        return self.setAttribute("alternative", alternative)

    def getTarget(self):
        return self.getAttribute("target")

    def setTarget(self, target):
        return self.setAttribute("target", target)

    def setFormularName(self, name):
        self.setAttribute("formularname", name)

    def getFormularName(self):
        return self.getAttribute("formularname")
    
    def setName(self, name):
        self.setAttribute("name", name)

    def getName(self):
        return self.getAttribute("name")

    def setDescription(self, description):
        self.setAttribute("description", description)

    def getDescription(self):
        return self.getAttribute("description")

    def getModes(self):
        return self.getAttribute("modes")

    def setModes(self, modes):
        self.setAttribute("modes", modes)

    def getFlags(self):
        return self.getAttribute("flags")

    def setFlags(self, flags):
        return self.setAttribute("flags", flags)

    def getParent(self):
        return self.parent

    def pathToRoot(self):
        current = self
        path = []
        while not current is None:
            path.insert(0, current)
            current = current.parent
        return path

    def depth(self):
        d = -1
        current = self
        while not current is None:
            d += 1
            current = current.parent
        return d

    def walk(self):
        yield self

    def visit(self, visitor):
        return visitor(self)

    def shallowCopy(self):
        nc = self.__class__()
        nc.attributes = self.attributes.copy()
        return nc

    def clone(self):
        return self.shallowCopy()

class Node(NodeComponent):

    def __init__(self):
        NodeComponent.__init__(self)
        self.attributes["containers"] = ""
        self.children = []

    def getContainers(self):
        return self.getAttribute("containers")

    def setContainers(self, containers):
        self.setAttribute("containers", containers)

    def addChild(self, child, idx=-1, broadcast=True):
        if idx == -1:
            idx = len(self.children)
        self.children.insert(idx, child)
        child.parent = self

    def indexOfChild(self, child):
        try:
            return self.children.index(child)
        except ValueError:
            return -1

    def getChildByName(self, name):
        for child in self.children:
            if child.getName() == name:
                return child
        return None

    def removeChild(self, child, broadcast=True):
        try:
            self.children.remove(child)
            child.parent = None
        except ValueError:
            return

    def walk(self):
        yield self
        for child in self.children:
            for x in child.walk():
                yield x

    def visit(self, visitor):

        result = visitor(self)
        if result == VISIT_IGNORE_CHILDREN:
            return VISIT_CONTINUE
        if result != VISIT_CONTINUE:
            return result
        for child in self.children:
            result = child.visit(visitor)
            if result == VISIT_IGNORE_CHILDREN:
                return VISIT_CONTINUE
            if result != VISIT_CONTINUE:
                return result
        return VISIT_CONTINUE

    def clone(self):
        n = NodeComponent.clone(self)
        children = []
        for child in self.children:
            child = child.clone()
            children.append(child)
            child.parent = n
        n.children = children
        return n

    def firstLevel(self, crit):
        visited = set()
        for c in self.walk():
            if c != self and crit(c):
                parent = c.parent
                found = False
                while parent:
                    if parent in visited:
                        found = True
                        break
                    parent = parent.parent
                if not found:
                    visited.add(c)
                    yield c

class RootNode(Node):
    def __init__(self, document=None):
        Node.__init__(self)
        self.document = document

class Leaf(NodeComponent):
    def __init__(self):
        NodeComponent.__init__(self)
        self.attributes["value"] = ""

    def getValue(self):
        return self.getAttribute("value")

    def setValue(self, value):
        self.setAttribute("value", value)

class Document:

    def __init__(self, root=None):
        if root is None: self.root = RootNode()
        else:            self.root = root
        self.root.document = self

    def importDocument(self, ndocument):
        root, nroot = self.root, ndocument.root
        if not root or not nroot: return

        for child in nroot.children[:]:
            nroot.removeChild(child)
            root.addChild(child)

    def getNames(self):
        names = set()
        root = self.root
        if not root: names
        for nc in root.walk():
            name = nc.getName()
            if name: names.add(name)
        return names

    def getNodeComponentByName(self, name):
        root = self.root
        if not root: None
        for child in root.walk():
            if child.getName() == name:
                return child
        return None

    def findByName(self, name):
        if self.root:
            for nc in self.root.walk():
                if name == nc.getName():
                    return nc
        return None

    def findByClassAndName(self, clazz, name=None):
        if self.root:
            for nc in self.root.walk():
                if isinstance(nc, clazz) and \
                    (not name or nc.getName() == name):
                    return nc
        return None

    def indexOfcomponentByClassAndName(self, clazz, name=None):
        idx = 0
        if self.root:
            for nc in self.root.walk():
                if isinstance(nc, clazz):
                    if not name or nc.getName() == name:
                        return idx
                    idx += 1
        return -1

    def findAllByClass(self, clazz):
        if self.root:
            for nc in self.root.walk():
                if isinstance(nc, clazz):
                    yield nc

    def walk(self):
        if self.root:
            for nc in self.root.walk():
                yield nc

# vim:set ts=4 sw=4 si et sta sts=4:
